package controller

import (
	"fmt"
	"strconv"
	"strings"
	"time"

	"be.ems/src/framework/constants/common"
	"be.ems/src/framework/i18n"
	"be.ems/src/framework/utils/ctx"
	"be.ems/src/framework/utils/file"
	"be.ems/src/framework/utils/parse"
	"be.ems/src/framework/vo/result"
	"be.ems/src/modules/system/model"
	"be.ems/src/modules/system/service"

	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
)

// 实例化控制层 SysDictTypeController 结构体
var NewSysDictType = &SysDictTypeController{
	sysDictTypeService: service.NewSysDictTypeImpl,
}

// 字典类型信息
//
// PATH /system/dict/type
type SysDictTypeController struct {
	// 字典类型服务
	sysDictTypeService service.ISysDictType
}

// 字典类型列表
//
// GET /list
func (s *SysDictTypeController) List(c *gin.Context) {
	querys := ctx.QueryMap(c)
	data := s.sysDictTypeService.SelectDictTypePage(querys)

	rows := data["rows"].([]model.SysDictType)
	// 闭包函数处理多语言
	language := ctx.AcceptLanguage(c)
	converI18n := func(language string, arr *[]model.SysDictType) {
		for i := range *arr {
			(*arr)[i].DictName = i18n.TKey(language, (*arr)[i].DictName)
			(*arr)[i].Remark = i18n.TKey(language, (*arr)[i].Remark)
		}
	}
	converI18n(language, &rows)

	c.JSON(200, result.Ok(data))
}

// 字典类型信息
//
// GET /:dictId
func (s *SysDictTypeController) Info(c *gin.Context) {
	language := ctx.AcceptLanguage(c)
	dictId := c.Param("dictId")
	if dictId == "" {
		c.JSON(400, result.CodeMsg(400, i18n.TKey(language, "app.common.err400")))
		return
	}
	data := s.sysDictTypeService.SelectDictTypeByID(dictId)
	if data.DictID == dictId {
		// 处理多语言
		data.DictName = i18n.TKey(language, data.DictName)
		data.Remark = i18n.TKey(language, data.Remark)
		c.JSON(200, result.OkData(data))
		return
	}
	c.JSON(200, result.Err(nil))
}

// 字典类型新增
//
// POST /
func (s *SysDictTypeController) Add(c *gin.Context) {
	language := ctx.AcceptLanguage(c)
	var body model.SysDictType
	err := c.ShouldBindBodyWith(&body, binding.JSON)
	if err != nil || body.DictID != "" {
		c.JSON(400, result.CodeMsg(400, i18n.TKey(language, "app.common.err400")))
		return
	}

	// 检查字典名称唯一
	uniqueDictName := s.sysDictTypeService.CheckUniqueDictName(body.DictName, "")
	if !uniqueDictName {
		// 字典新增【%s】失败，字典名称已存在
		msg := i18n.TTemplate(language, "dictType.errNameExists", map[string]any{"name": body.DictName})
		c.JSON(200, result.ErrMsg(msg))
		return
	}

	// 检查字典类型唯一
	uniqueDictType := s.sysDictTypeService.CheckUniqueDictType(body.DictType, "")
	if !uniqueDictType {
		// 字典新增【%s】失败，字典类型已存在
		msg := i18n.TTemplate(language, "dictType.errTypeExists", map[string]any{"name": body.DictName})
		c.JSON(200, result.ErrMsg(msg))
		return
	}

	body.CreateBy = ctx.LoginUserToUserName(c)
	insertId := s.sysDictTypeService.InsertDictType(body)
	if insertId != "" {
		c.JSON(200, result.Ok(nil))
		return
	}
	c.JSON(200, result.Err(nil))
}

// 字典类型修改
//
// PUT /
func (s *SysDictTypeController) Edit(c *gin.Context) {
	language := ctx.AcceptLanguage(c)
	var body model.SysDictType
	err := c.ShouldBindBodyWith(&body, binding.JSON)
	if err != nil || body.DictID == "" {
		c.JSON(400, result.CodeMsg(400, i18n.TKey(language, "app.common.err400")))
		return
	}

	// 检查数据是否存在
	dictInfo := s.sysDictTypeService.SelectDictTypeByID(body.DictID)
	if dictInfo.DictID != body.DictID {
		// 没有可访问字典类型数据！
		c.JSON(200, result.ErrMsg(i18n.TKey(language, "dictType.noData")))
		return
	}

	// 检查字典名称唯一
	uniqueDictName := s.sysDictTypeService.CheckUniqueDictName(body.DictName, body.DictID)
	if !uniqueDictName {
		// 字典修改【%s】失败，字典名称已存在
		msg := i18n.TTemplate(language, "dictType.errNameExists", map[string]any{"name": body.DictName})
		c.JSON(200, result.ErrMsg(msg))
		return
	}

	// 检查字典类型唯一
	uniqueDictType := s.sysDictTypeService.CheckUniqueDictType(body.DictType, body.DictID)
	if !uniqueDictType {
		// 字典修改【%s】失败，字典类型已存在
		msg := i18n.TTemplate(language, "dictType.errTypeExists", map[string]any{"name": body.DictName})
		c.JSON(200, result.ErrMsg(msg))
		return
	}

	// 多语言非原始值
	i18nValue := i18n.TKey(language, dictInfo.DictName)
	if i18nValue != dictInfo.DictName {
		i18n.UpdateKeyValue(language, dictInfo.DictName, body.DictName)
		body.DictName = dictInfo.DictName
	}
	// 多语言非原始值
	i18nValue2 := i18n.TKey(language, dictInfo.Remark)
	if i18nValue2 != dictInfo.Remark {
		i18n.UpdateKeyValue(language, dictInfo.Remark, body.Remark)
		body.Remark = dictInfo.Remark
	}

	body.UpdateBy = ctx.LoginUserToUserName(c)
	rows := s.sysDictTypeService.UpdateDictType(body)
	if rows > 0 {
		c.JSON(200, result.Ok(nil))
		return
	}
	c.JSON(200, result.Err(nil))
}

// 字典类型删除
//
// DELETE /:dictIds
func (s *SysDictTypeController) Remove(c *gin.Context) {
	language := ctx.AcceptLanguage(c)
	dictIds := c.Param("dictIds")
	if dictIds == "" {
		c.JSON(400, result.CodeMsg(400, i18n.TKey(language, "app.common.err400")))
		return
	}
	// 处理字符转id数组后去重
	ids := strings.Split(dictIds, ",")
	uniqueIDs := parse.RemoveDuplicates(ids)
	if len(uniqueIDs) <= 0 {
		c.JSON(200, result.Err(nil))
		return
	}
	rows, err := s.sysDictTypeService.DeleteDictTypeByIDs(uniqueIDs)
	if err != nil {
		c.JSON(200, result.ErrMsg(err.Error()))
		return
	}
	msg := i18n.TTemplate(language, "app.common.deleteSuccess", map[string]any{"num": rows})
	c.JSON(200, result.OkMsg(msg))
}

// 字典类型刷新缓存
//
// PUT /refreshCache
func (s *SysDictTypeController) RefreshCache(c *gin.Context) {
	s.sysDictTypeService.ResetDictCache()
	i18n.ClearLocaleData() // 清空国际化数据
	c.JSON(200, result.Ok(nil))
}

// 字典类型选择框列表
//
// GET /getDictOptionselect
func (s *SysDictTypeController) DictOptionselect(c *gin.Context) {
	data := s.sysDictTypeService.SelectDictTypeList(model.SysDictType{
		Status: common.STATUS_YES,
	})

	type labelValue struct {
		Label string `json:"label"`
		Value string `json:"value"`
	}

	language := ctx.AcceptLanguage(c)
	// 数据组
	arr := []labelValue{}
	for _, v := range data {
		arr = append(arr, labelValue{
			Label: i18n.TKey(language, v.DictName),
			Value: v.DictType,
		})
	}
	c.JSON(200, result.OkData(arr))
}

// 字典类型列表导出
//
// POST /export
func (s *SysDictTypeController) Export(c *gin.Context) {
	language := ctx.AcceptLanguage(c)
	// 查询结果，根据查询条件结果，单页最大值限制
	querys := ctx.BodyJSONMap(c)
	data := s.sysDictTypeService.SelectDictTypePage(querys)
	if data["total"].(int64) == 0 {
		// 导出数据记录为空
		c.JSON(200, result.ErrMsg(i18n.TKey(language, "app.common.exportEmpty")))
		return
	}
	rows := data["rows"].([]model.SysDictType)

	// 闭包函数处理多语言
	converI18n := func(language string, arr *[]model.SysDictType) {
		for i := range *arr {
			(*arr)[i].DictName = i18n.TKey(language, (*arr)[i].DictName)
			(*arr)[i].Remark = i18n.TKey(language, (*arr)[i].Remark)
		}
	}
	converI18n(language, &rows)

	// 导出文件名称
	fileName := fmt.Sprintf("dict_type_export_%d_%d.xlsx", len(rows), time.Now().UnixMilli())
	// 第一行表头标题
	headerCells := map[string]string{
		"A1": i18n.TKey(language, "dictType.export.id"),
		"B1": i18n.TKey(language, "dictType.export.name"),
		"C1": i18n.TKey(language, "dictType.export.type"),
		"D1": i18n.TKey(language, "dictType.export.status"),
	}
	// 从第二行开始的数据
	dataCells := make([]map[string]any, 0)
	for i, row := range rows {
		idx := strconv.Itoa(i + 2)
		statusValue := i18n.TKey(language, "dictData.disable")
		if row.Status == "1" {
			statusValue = i18n.TKey(language, "dictData.normal")
		}
		dataCells = append(dataCells, map[string]any{
			"A" + idx: row.DictID,
			"B" + idx: row.DictName,
			"C" + idx: row.DictType,
			"D" + idx: statusValue,
		})
	}

	// 导出数据表格
	saveFilePath, err := file.WriteSheet(headerCells, dataCells, fileName, "")
	if err != nil {
		c.JSON(200, result.ErrMsg(err.Error()))
		return
	}

	c.FileAttachment(saveFilePath, fileName)
}
